import datetime
import pandas
from pyrror.DiscrepancyMonitoring.DiscrepancyMonitoring import get_alignment_config


def send(discrepancy_monitoring, query_result, source):
    table_name = discrepancy_monitoring.code
    schema_name = "__monitoring"

    if not query_result:
        return 0

    __dict_added__ = {
        "source": source,
        "created_at": datetime.datetime.now()
    }

    dict_columns = query_result[0]
    dict_columns.update(__dict_added__)
    columns_name = list(dict_columns.keys())

    rows = []
    for r in query_result:
        r.update(__dict_added__)
        row = []
        for c in columns_name:
            row.append(r[c])
        rows.append(row)

    monitoring_data = {
        "table_name": "%s.%s" % (schema_name, table_name),
        "columns_name": columns_name,
        "rows": rows
    }
    discrepancy_monitoring.dbstream.send_data(monitoring_data, replace=False)


def collect_from_query(discrepancy_monitoring, query_result, source):
    _alignment = get_alignment_config(discrepancy_monitoring)
    _mapping = _alignment.get("sources").get(source).get("mapping")
    df = pandas.DataFrame(query_result)
    df = df.rename(columns=_mapping)
    _filter = _alignment.get("filter")
    if _filter:
        df = df[eval(_filter.strip())]
    _list_to_group = []
    for d in _alignment["dimensions"]:
        if d in df.columns:
            _list_to_group.append(d)
    if _list_to_group:
        df = df.groupby(_list_to_group, as_index=False).sum()

        _data = df.to_dict(orient="records")
    else:
        _data = df.sum()

    send(discrepancy_monitoring=discrepancy_monitoring, query_result=_data, source=source)



import requests

from pyrror.DiscrepancyMonitoring.analyze import analyze
from pyrror.DiscrepancyMonitoring.collect import collect_from_query
from pyrror.Error import Error


def get_alignment_config(discrepancy_monitoring):
    r = requests.get("%s/%s/%s/config" % (discrepancy_monitoring.monitoring_discrepancy_url,
                                          discrepancy_monitoring.client_id,
                                          discrepancy_monitoring.cod)).json()
    if r.get("error"):
        discrepancy_monitoring.error.log_error(function_name="get_alignment_config", error=r.get("error"))
    return r.get("config")


class DiscrepancyMonitoring:
    def __init__(self, dbstream, client_id, discrepancy_code, monitoring_url):
        self.client_id = client_id
        self.dbstream = dbstream
        self.code = discrepancy_code
        self.monitoring_url = monitoring_url
        self.monitoring_discrepancy_url = monitoring_url + "/api/monitoring/discrepancy/"
        self.error = Error(
            monitoring_error_url=monitoring_url + "/api/monitoring/error_to_slack/",
            client_id=client_id
        )

    def collect_from_query(self, query_result, source):
        collect_from_query(self, query_result, source)

    def analyze(self):
        analyze(self)



import json
import os
import requests
from pyrror.DiscrepancyMonitoring.DiscrepancyMonitoring import get_alignment_config


def get_test(discrepancy_monitoring, _alignment):
    dimensions_string = ",".join(_alignment["dimensions"])
    comparison_metrics = [
        "lag(%(metric)s) over (partition by %(dimensions_string)s order by source) as comparison_%(metric)s"
        % {"metric": metric, 'dimensions_string': dimensions_string}
        for metric in _alignment["metrics"]
    ]
    comparison_metrics_string = ",".join(comparison_metrics)

    check_metrics = [
        "(%(metric)s - comparison_%(metric)s)::float/comparison_%(metric)s as check_%(metric)s"
        % {"metric": metric}
        for metric in _alignment["metrics"]
    ]
    check_metrics_string = ",".join(check_metrics)

    query = """
    with inter as (SELECT
    *,
           rank() over (partition by source order by created_at desc) as rom_numb
    FROM __monitoring.%(alignment)s),
         inter2 as (
             select *,
                    lag(source) over (partition by %(dimensions_string)s order by source) as comparison_source,
                    %(comparison_metrics_string)s

             from inter
             where rom_numb = 1
         )
    select 
    %(check_metrics_string)s 
    , * from inter2
    where comparison_source is not null;
    """ % {
        "alignment": discrepancy_monitoring.code,
        "dimensions_string": dimensions_string,
        "comparison_metrics_string": comparison_metrics_string,
        "check_metrics_string": check_metrics_string
    }
    return discrepancy_monitoring.dbstream.execute_query(query)


def analyze(discrepancy_monitoring):
    _alignment = get_alignment_config(discrepancy_monitoring)
    _test = get_test(discrepancy_monitoring, _alignment)
    for element in _test:
        for metric in _alignment["metrics"]:
            if element["check_" + metric] > 0:
                dimensions = {}
                for d in _alignment["dimensions"]:
                    dimensions.update({d: element[d]})
                _error_dict = {
                    "client_id": discrepancy_monitoring.client,
                    "code": discrepancy_monitoring.code,
                    "source": element["source"],
                    "comparison_source": element["comparison_source"],
                    "metric": metric,
                    "error": element["check_" + metric],
                    "dimensions": dimensions,
                }
                requests.post(url=os.environ["MONITORING_DISCREPANCY_URL"], data=json.dumps(_error_dict, default=str))



import datetime
import json
import os
import traceback
import requests


class Error:

    def __init__(self, client_id, monitoring_error_url):
        self.client_id = client_id
        self.monitoring_error_url = monitoring_error_url

    def log(self, function_name, error=None):
        body = {
            "client_id": self.client_id,
            "timestamp": str(datetime.datetime.now()),
            "local_absolute_path": os.getcwd(),
            "function_name": function_name,
            "error": traceback.format_exc() if error is None else error
        }
        r = requests.post(url=self.monitoring_error_url, data=json.dumps(body))
        return r.status_code

    def log_error(self, function_name, error):
        body = {
            "client_id": self.client_id,
            "timestamp": str(datetime.datetime.now()),
            "local_absolute_path": os.getcwd(),
            "function_name": function_name,
            "error": error
        }
        r = requests.post(url=self.monitoring_error_url, data=json.dumps(body))
        return r.status_code



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyrror",
    version="0.0.3",
    author="Dacker",
    author_email="hello@dacker.co",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dacker-team/pyrror",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3',
    install_requires=[
        "requests==2.22.0",
        "pandas==1.0.3",
        "pyyaml==5.3.1"
    ],
)
